@@ -18,9 +18,7 @@
 See also: https://github.com/textile/python-textile
 """
 
-from StringIO import StringIO
-from genshi.core import Stream
-from genshi.input import HTMLParser, ParseError
+from HTMLParser import HTMLParseError
 
 try:
     import textile
@@ -28,8 +26,8 @@ except ImportError:
     textile = None
 has_textile = textile is not None
 
+from trac.api import ISystemInfoProvider
 from trac.core import Component, implements
-from trac.env import ISystemInfoProvider
 from trac.mimeview.api import IHTMLPreviewRenderer
 from trac.util import get_pkginfo, lazy
 from trac.util.html import Markup, TracHTMLSanitizer, escape
@@ -74,9 +72,8 @@ class TextileRenderer(Component):
         if WikiSystem(self.env).render_unsafe_content:
             return Markup(output)
         try:
-            stream = Stream(HTMLParser(StringIO(output)))
-            return (stream | self._sanitizer).render('xhtml', encoding=None)
-        except ParseError as e:
+            return self._sanitizer.sanitize(output)
+        except HTMLParseError as e:
             self.log.warning(e)
             line = content.splitlines()[e.lineno - 1].strip()
             return system_message(_("HTML parsing error: %(message)s",
