@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2019 Edgewall Software
+# Copyright (C) 2008-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -138,8 +138,12 @@ class FunctionalTestEnvironment(object):
 
         config_file = os.path.join(self.dirname, 'config.ini')
         config = Configuration(config_file)
-        config.set('repositories', '.dir', self.repo_path_for_initenv())
-        config.set('repositories', '.type', self.repotype)
+        repo_path = self.repo_path_for_initenv()
+        if repo_path:
+            config.set('repositories', '.dir', repo_path)
+            config.set('repositories', '.type', self.repotype)
+        for component in self.get_enabled_components():
+            config.set('components', component, 'enabled')
         config.save()
         self._tracadmin('initenv', self.tracdir, self.dburi,
                         '--config=%s' % config_file)
@@ -152,9 +156,6 @@ class FunctionalTestEnvironment(object):
         self.adduser('joe')
         self.grant_perm('admin', 'TRAC_ADMIN')
         env = self.get_trac_environment()
-        for component in self.get_enabled_components():
-            env.config.set('components', component, 'enabled')
-        env.config.save()
         self.post_create(env)
 
     def adduser(self, user):
@@ -203,25 +204,25 @@ class FunctionalTestEnvironment(object):
         self.get_trac_environment().config.touch()
 
     def set_config(self, *args):
-        """Calls trac-admin to get the value for the given option
+        """Calls trac-admin to set the value for the given option
         in `trac.ini`."""
         self._tracadmin('config', 'set', *args)
 
     def get_config(self, *args):
-        """Calls trac-admin to set the value for the given option
+        """Calls trac-admin to get the value for the given option
         in `trac.ini`."""
         return self._tracadmin('config', 'get', *args)
 
     def remove_config(self, *args):
         """Calls trac-admin to remove the value for the given option
         in `trac.ini`."""
-        return self._tracadmin('config', 'remove', *args)
+        self._tracadmin('config', 'remove', *args)
 
     def _tracadmin(self, *args):
         """Internal utility method for calling trac-admin"""
         proc = Popen([sys.executable, os.path.join(self.trac_src, 'trac',
                       'admin', 'console.py'), self.tracdir],
-                     stdin=PIPE, stdout=PIPE, stderr=STDOUT,
+                     stdin=PIPE, stdout=PIPE, stderr=PIPE,
                      close_fds=close_fds, cwd=self.command_cwd)
         if args:
             if any('\n' in arg for arg in args):
@@ -233,13 +234,12 @@ class FunctionalTestEnvironment(object):
                              for idx, arg in enumerate(args))
         else:
             input = None
-        out = proc.communicate(input=input)[0]
-        if proc.returncode:
-            print(out)
-            self.logfile.write(out)
+        out, err = proc.communicate(input=input)
+        if proc.returncode or err:
+            self.logfile.write(err)
             raise Exception("Failed while running trac-admin with arguments %r.\n"
                             "Exitcode: %s \n%s"
-                            % (args, proc.returncode, out))
+                            % (args, proc.returncode, err))
         else:
             # trac-admin is started in interactive mode, so we strip away
             # everything up to the to the interactive prompt
@@ -247,20 +247,14 @@ class FunctionalTestEnvironment(object):
 
     def start(self):
         """Starts the webserver, and waits for it to come up."""
-        if 'FIGLEAF' in os.environ:
-            exe = os.environ['FIGLEAF']
-            if ' ' in exe: # e.g. 'coverage run'
-                args = exe.split()
-            else:
-                args = [exe]
-        else:
-            args = [sys.executable]
+        args = [
+            sys.executable,
+            os.path.join(self.trac_src, 'trac', 'web', 'standalone.py')
+        ]
         options = ["--port=%s" % self.port, "-s", "--hostname=127.0.0.1",
                    "--basic-auth=trac,%s," % self.htpasswd]
         if 'TRAC_TEST_TRACD_OPTIONS' in os.environ:
             options += os.environ['TRAC_TEST_TRACD_OPTIONS'].split()
-        args.append(os.path.join(self.trac_src, 'trac', 'web',
-                                 'standalone.py'))
         server = Popen(args + options + [self.tracdir],
                        stdout=self.logfile, stderr=self.logfile,
                        close_fds=close_fds,
@@ -298,7 +292,7 @@ class FunctionalTestEnvironment(object):
 
     def repo_path_for_initenv(self):
         """Default to no repository"""
-        return ''
+        return None
 
     def call_in_dir(self, dir, args, environ=None):
         proc = Popen(args, stdout=PIPE, stderr=self.logfile,
