@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2019 Edgewall Software
+# Copyright (C) 2004-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -1434,7 +1434,7 @@ class QueryLinksTestCase(unittest.TestCase):
     def test_duplicated_report_arguments(self):
         data = self._process_request('report=1&report=2')
         self.assertEqual([], data['tickets'])
-        self.assertEqual('1', data['query'].id)
+        self.assertEqual(1, data['query'].id)
 
     def test_duplicated_group_arguments(self):
         self._insert_ticket(status='new')
@@ -1475,6 +1475,16 @@ class QueryLinksTestCase(unittest.TestCase):
         self.assertTrue(data['query'].groupdesc)
         self.assertFalse(data['query'].desc)
 
+    def test_non_numeric_report_argument(self):
+        data = self._process_request('id=0&report=42xx')
+        self.assertEqual([], data['tickets'])
+        self.assertEqual(None, data['query'].id)
+
+    def test_unicode_report_argument(self):
+        data = self._process_request(u'id=0&report=42éá')
+        self.assertEqual([], data['tickets'])
+        self.assertEqual(None, data['query'].id)
+
 
 class TicketQueryMacroTestCase(unittest.TestCase):
 
@@ -1660,7 +1670,7 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table summary="Ticket completion status for each project">
+  <table>
     <tr>
       <th scope="row">
         <i><a href="/query?project=&amp;max=0&amp;order=id">(none)</a></i>
@@ -1711,7 +1721,7 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table summary="Ticket completion status for each project">
+  <table>
     <tr>
       <th scope="row">
         <a href="/query?project=xmas&amp;max=0&amp;order=id">xmas</a>
@@ -1762,7 +1772,7 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table summary="Ticket completion status for each project">
+  <table>
     <tr>
       <th scope="row">
         <a href="/query?project=xmas&amp;reporter=santa&amp;max=0&amp;order=id">xmas</a>
@@ -1792,7 +1802,7 @@ QUERY_TEST_CASES = u"""
 ------------------------------
 <p>
 </p><div class="trac-groupprogress">
-  <table summary="Ticket completion status for each project">
+  <table>
     <tr>
       <th scope="row">
         <a href="/query?project=xmas&amp;reporter=santa&amp;or&amp;owner=santa&amp;project=xmas&amp;max=0&amp;order=id">xmas</a>
