@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2014-2019 Edgewall Software
+# Copyright (C) 2014-2020 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -19,7 +19,8 @@ from trac.core import Component, TracError, implements
 from trac.perm import PermissionCache, PermissionSystem
 from trac.resource import Resource, ResourceNotFound
 from trac.test import EnvironmentStub, MockRequest
-from trac.ticket.api import ITicketActionController, TicketSystem
+from trac.ticket.api import (ITicketActionController, ITicketManipulator,
+                             TicketSystem)
 from trac.ticket.model import Milestone, Ticket, Version
 from trac.ticket.test import insert_ticket
 from trac.ticket.web_ui import DefaultTicketPolicy, TicketModule
@@ -32,7 +33,9 @@ from trac.web.chrome import Chrome
 
 class TicketModuleTestCase(unittest.TestCase):
 
+    mock_components = None
     mock_ticket_operation = None
+    mock_ticket_manipulator = None
 
     @classmethod
     def setUpClass(cls):
@@ -60,12 +63,34 @@ class TicketModuleTestCase(unittest.TestCase):
                 if action == 'mock':
                     self.side_effect_count += 1
 
+        class MockTicketManipulator(Component):
+
+            implements(ITicketManipulator)
+
+            def __init__(self):
+                self.validate_ticket_called = 0
+                self.validate_comment_called = 0
+
+            def prepare_ticket(self, req, ticket, fields, actions):
+                pass
+
+            def validate_ticket(self, req, ticket):
+                self.validate_ticket_called += 1
+                return []
+
+            def validate_comment(self, req, comment):
+                self.validate_comment_called += 1
+                return []
+
         cls.mock_ticket_operation = MockTicketOperation
+        cls.mock_ticket_manipulator = MockTicketManipulator
+        cls.mock_components = (MockTicketOperation, MockTicketManipulator)
 
     @classmethod
     def tearDownClass(cls):
         from trac.core import ComponentMeta
-        ComponentMeta.deregister(cls.mock_ticket_operation)
+        for m in cls.mock_components:
+            ComponentMeta.deregister(m)
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
@@ -97,6 +122,55 @@ class TicketModuleTestCase(unittest.TestCase):
         """Helper for inserting a ticket into the database"""
         return insert_ticket(self.env, **kw)
 
+    def _prepare_newticket_post_request(self):
+        self.env.config.set('ticket', 'default_component', 'component1')
+        self.env.config.set('ticket', 'default_milestone', 'milestone1')
+        req = MockRequest(self.env, method='GET', path_info='/newticket',
+                          authname='user1')
+
+        mod = TicketModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        data = mod.process_request(req)[1]
+
+        fields = {'field_%s' % f['name']: f.get('value', '')
+                  for f in data['fields']}
+        fields['field_summary'] = 'The summary'
+        req = MockRequest(
+            self.env, method='POST', path_info='/newticket', authname='user1',
+            args=dict(action=data['action'], submit='Create ticket', **fields)
+        )
+        return req
+
+    def _prepare_ticket_post_request(self, tid):
+        path_info = '/ticket/%s' % tid
+        req = MockRequest(self.env, method='GET', path_info=path_info,
+                          authname='user1')
+
+        mod = TicketModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        data = mod.process_request(req)[1]
+
+        start_time = to_utimestamp(data['start_time'])
+        ticket = data['ticket']
+        fields = {'field_%s' % f: v for f, v in ticket.values.iteritems()}
+        req = MockRequest(
+            self.env, method='POST', path_info=path_info, authname='user1',
+            args=dict(action=data['action'], submit='Submit changes',
+                      comment='', replyto='', start_time=start_time,
+                      view_time=start_time, **fields)
+        )
+        return req
+
+    def _process_ticket_request(self, req):
+        mod = TicketModule(self.env)
+        self.assertTrue(mod.match_request(req))
+        try:
+            data = mod.process_request(req)[1]
+        except RequestDone:
+            return None
+        else:
+            return data
+
     def _get_field_by_name(self, data, name):
         for field in data['fields']:
             if field['name'] == name:
@@ -182,6 +256,36 @@ class TicketModuleTestCase(unittest.TestCase):
         self.assertNotIn(u'title="Released ',
                          unicode(version_field(data)['rendered']))
 
+    def test_comment_save(self):
+        req = self._prepare_newticket_post_request()
+        self._process_ticket_request(req)
+
+        req = self._prepare_ticket_post_request(1)
+        req.args['comment'] = 'The comment'
+        data = self._process_ticket_request(req)
+
+        self.assertIsNone(data)
+        self.assertEqual([], req.chrome['warnings'])
+        self.assertEqual(['303 See Other'], req.status_sent)
+        self.assertEqual('http://example.org/trac.cgi/ticket/1#comment:1',
+                         req.headers_sent['Location'])
+        self.assertEqual(1, len(Ticket(self.env, 1).get_changelog()))
+
+    def test_whitespace_comment_not_saved(self):
+        req = self._prepare_newticket_post_request()
+        self._process_ticket_request(req)
+
+        req = self._prepare_ticket_post_request(1)
+        req.args['comment'] = u'\u200b\t\t\n\n\n\n\u200b'
+        data = self._process_ticket_request(req)
+
+        self.assertIsNone(data)
+        self.assertEqual([], req.chrome['warnings'])
+        self.assertEqual(['303 See Other'], req.status_sent)
+        self.assertEqual('http://example.org/trac.cgi/ticket/1',
+                         req.headers_sent['Location'])
+        self.assertEqual(0, len(Ticket(self.env, 1).get_changelog()))
+
     def test_quoted_reply_author_is_obfuscated(self):
         """Reply-to author is obfuscated in a quoted reply."""
         author = 'author <author@example.net>'
@@ -775,7 +879,7 @@ class TicketModuleTestCase(unittest.TestCase):
         """New ticket page has autopreview."""
         req = MockRequest(self.env, method='GET', path_info='/newticket')
 
-        self.assertTrue(self.ticket_module.process_request(req))
+        self.assertTrue(self.ticket_module.match_request(req))
         self.ticket_module.process_request(req)
 
         self.assertTrue(self._has_auto_preview(req))
@@ -787,7 +891,7 @@ class TicketModuleTestCase(unittest.TestCase):
         config.remove('ticket-workflow', 'create_and_assign')
         req = MockRequest(self.env, method='GET', path_info='/newticket')
 
-        self.assertTrue(self.ticket_module.process_request(req))
+        self.assertTrue(self.ticket_module.match_request(req))
         data = self.ticket_module.process_request(req)[1]
 
         self.assertEqual([], data['action_controls'])
@@ -800,9 +904,10 @@ class TicketModuleTestCase(unittest.TestCase):
         for option in config.options('ticket-workflow'):
             if not option[0].startswith('leave'):
                 config.remove('ticket-workflow', option[0])
+        self._insert_ticket(summary='the summary')
         req = MockRequest(self.env, method='GET', path_info='/ticket/1')
 
-        self.assertTrue(self.ticket_module.process_request(req))
+        self.assertTrue(self.ticket_module.match_request(req))
         data = self.ticket_module.process_request(req)[1]
 
         self.assertEqual([], data['action_controls'])
@@ -939,6 +1044,19 @@ class TicketModuleTestCase(unittest.TestCase):
                   'start_time': when_ts, 'view_time': when_ts})
         self.assertEqual('2009-02-13T23:31:30Z', old_values['due'])
 
+    def test_newticket_ticket_validate_comment_not_called(self):
+        req = self._prepare_newticket_post_request()
+
+        self._process_ticket_request(req)
+
+        self.assertEqual([], req.chrome['warnings'])
+        self.assertEqual(['303 See Other'], req.status_sent)
+        self.assertEqual('http://example.org/trac.cgi/ticket/1',
+                         req.headers_sent['Location'])
+        tm = self.mock_ticket_manipulator(self.env)
+        self.assertEqual(0, tm.validate_comment_called)
+        self.assertEqual(1, tm.validate_ticket_called)
+
     def _reset_ticket_fields(self):
         tktsys = TicketSystem(self.env)
         tktsys.reset_ticket_fields()
@@ -949,7 +1067,7 @@ class CustomFieldMaxSizeTestCase(unittest.TestCase):
     """Tests for [ticket-custom] max_size attribute."""
 
     def setUp(self):
-        self.env = EnvironmentStub()
+        self.env = EnvironmentStub(default_data=True)
         self.ticket_module = TicketModule(self.env)
 
     def tearDown(self):
@@ -958,13 +1076,14 @@ class CustomFieldMaxSizeTestCase(unittest.TestCase):
     def _setup_env_and_req(self, max_size, field_value):
         self.env.config.set('ticket-custom', 'text1', 'text')
         self.env.config.set('ticket-custom', 'text1.max_size', max_size)
-        ticket = insert_ticket(self.env, summary='summary', text1='init')
+        ticket = insert_ticket(self.env, text1='init')
         change_time = Ticket(self.env, ticket.id)['changetime']
         view_time = str(to_utimestamp(change_time))
         req = MockRequest(
             self.env, method='POST', path_info='/ticket/%d' % ticket.id,
             args={'submit': 'Submit changes', 'field_text1': field_value,
-                  'action': 'leave', 'view_time': view_time})
+                  'action': 'leave', 'view_time': view_time,
+                  'start_time': view_time})
         return req
 
     def test_ticket_custom_field_greater_than_max_size(self):
