@@ -21,22 +21,22 @@ import pkg_resources
 import re
 from datetime import datetime, timedelta
 
-from genshi.builder import tag
-
 from trac.config import IntOption, BoolOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.timeline.api import ITimelineEventProvider
 from trac.util.datefmt import (datetime_now, format_date, format_datetime,
                                format_time, localtz, parse_date,
-                               pretty_timedelta, utc, to_datetime,
-                               to_utimestamp, user_time)
-from trac.util.text import exception_to_unicode, to_unicode
-from trac.util.translation import _, tag_
+                               pretty_timedelta, to_datetime, to_utimestamp,
+                               truncate_datetime, user_time, utc)
+from trac.util.html import tag
+from trac.util.text import to_unicode
+from trac.util.translation import _
 from trac.web import IRequestHandler, IRequestFilter
-from trac.web.chrome import (Chrome, INavigationContributor,
-                             ITemplateProvider, add_link, add_stylesheet,
-                             add_warning, auth_link, prevnext_nav, web_context)
+from trac.web.chrome import (Chrome, INavigationContributor, ITemplateProvider,
+                             accesskey, add_link, add_stylesheet, add_warning,
+                             auth_link, component_guard, prevnext_nav,
+                             web_context)
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.formatter import concat_path_query_fragment, \
                                 split_url_into_path_query_fragment
@@ -76,7 +76,8 @@ class TimelineModule(Component):
     def get_navigation_items(self, req):
         if 'TIMELINE_VIEW' in req.perm('timeline'):
             yield ('mainnav', 'timeline',
-                   tag.a(_("Timeline"), href=req.href.timeline(), accesskey=2))
+                   tag.a(_("Timeline"), href=req.href.timeline(),
+                         accesskey=accesskey(req, 2)))
 
     # IPermissionRequestor methods
 
@@ -91,7 +92,7 @@ class TimelineModule(Component):
     def process_request(self, req):
         req.perm('timeline').require('TIMELINE_VIEW')
 
-        format = req.args.getfirst('format')
+        format = req.args.get('format')
         maxrows = req.args.getint('max', 50 if format == 'rss' else 0)
         lastvisit = req.session.as_int('timeline.lastvisit', 0)
 
@@ -104,13 +105,14 @@ class TimelineModule(Component):
 
         # Parse the from date and adjust the timestamp to the last second of
         # the day
-        fromdate = today = datetime_now(req.tz)
+        fromdate = datetime_now(req.tz)
+        today = truncate_datetime(fromdate)
         yesterday = to_datetime(today.replace(tzinfo=None) - timedelta(days=1),
                                 req.tz)
         precisedate = precision = None
         if 'from' in req.args:
             # Acquire from date only from non-blank input
-            reqfromdate = req.args.getfirst('from').strip()
+            reqfromdate = req.args.get('from').strip()
             if reqfromdate:
                 try:
                     precisedate = user_time(req, parse_date, reqfromdate)
@@ -118,7 +120,7 @@ class TimelineModule(Component):
                     add_warning(req, e)
                 else:
                     fromdate = precisedate.astimezone(req.tz)
-            precision = req.args.getfirst('precision', '')
+            precision = req.args.get('precision', '')
             if precision.startswith('second'):
                 precision = timedelta(seconds=1)
             elif precision.startswith('minute'):
@@ -136,23 +138,22 @@ class TimelineModule(Component):
         daysback = req.args.as_int('daysback', default,
                                    min=1, max=self.max_daysback)
 
-        authors = req.args.getfirst('authors')
+        authors = req.args.get('authors')
         if authors is None and format != 'rss':
             authors = req.session.get('timeline.authors')
         authors = (authors or '').strip()
 
         data = {'fromdate': fromdate, 'daysback': daysback,
-                'authors': authors,
-                'today': user_time(req, format_date, today),
-                'yesterday': user_time(req, format_date, yesterday),
+                'authors': authors, 'today': today, 'yesterday': yesterday,
                 'precisedate': precisedate, 'precision': precision,
                 'events': [], 'filters': [],
-                'abbreviated_messages': self.abbreviated_messages,
-                'lastvisit': lastvisit}
+                'abbreviated_messages': self.abbreviated_messages}
 
         available_filters = []
         for event_provider in self.event_providers:
-            available_filters += event_provider.get_timeline_filters(req) or []
+            with component_guard(self.env, req, event_provider):
+                available_filters += (event_provider.get_timeline_filters(req)
+                                      or [])
 
         # check the request or session for enabled filters, or use default
         filters = [f[0] for f in available_filters if f[0] in req.args]
@@ -188,19 +189,17 @@ class TimelineModule(Component):
         # gather all events for the given period of time
         events = []
         for provider in self.event_providers:
-            try:
+            with component_guard(self.env, req, provider):
                 for event in provider.get_timeline_events(req, start, stop,
                                                           filters) or []:
                     author = (event[2] or '').lower()
-                    if (not include or author in include) \
-                            and author not in exclude:
-                        events.append(self._event_data(provider, event))
-            except Exception as e:  # cope with a failure of that provider
-                self._provider_failure(e, req, provider, filters,
-                                       [f[0] for f in available_filters])
+                    if ((not include or author in include) and
+                        author not in exclude):
+                        events.append(
+                            self._event_data(req, provider, event, lastvisit))
 
         # prepare sorted global list
-        events = sorted(events, key=lambda e: e['date'], reverse=True)
+        events = sorted(events, key=lambda e: e['datetime'], reverse=True)
         if maxrows:
             events = events[:maxrows]
 
@@ -210,14 +209,14 @@ class TimelineModule(Component):
             rss_context = web_context(req, absurls=True)
             rss_context.set_hints(wiki_flavor='html', shorten_lines=False)
             data['context'] = rss_context
-            return 'timeline.rss', data, 'application/rss+xml'
+            return 'timeline.rss', data, {'content_type': 'application/rss+xml'}
         else:
             req.session.set('timeline.daysback', daysback,
                             self.default_daysback)
             req.session.set('timeline.authors', authors, '')
             # store lastvisit
             if events and not revisit:
-                lastviewed = to_utimestamp(events[0]['date'])
+                lastviewed = to_utimestamp(events[0]['datetime'])
                 req.session['timeline.lastvisit'] = max(lastvisit, lastviewed)
                 req.session['timeline.nextlastvisit'] = lastvisit
             html_context = web_context(req)
@@ -257,7 +256,7 @@ class TimelineModule(Component):
                      _("Next Period"))
         prevnext_nav(req, _("Previous Period"), _("Next Period"))
 
-        return 'timeline.html', data, None
+        return 'timeline.html', data
 
     # ITemplateProvider methods
 
@@ -272,7 +271,7 @@ class TimelineModule(Component):
     def pre_process_request(self, req, handler):
         return handler
 
-    def post_process_request(self, req, template, data, content_type):
+    def post_process_request(self, req, template, data, metadata):
         if data:
             def pretty_dateinfo(date, format=None, dateonly=False):
                 if not date:
@@ -321,7 +320,7 @@ class TimelineModule(Component):
                 return pretty_dateinfo(date, format='relative', dateonly=True)
             data['pretty_dateinfo'] = pretty_dateinfo
             data['dateinfo'] = dateinfo
-        return template, data, content_type
+        return template, data, metadata
 
     # IWikiSyntaxProvider methods
 
@@ -363,49 +362,20 @@ class TimelineModule(Component):
 
     # Internal methods
 
-    def _event_data(self, provider, event):
+    def _event_data(self, req, provider, event, lastvisit):
         """Compose the timeline event date from the event tuple and prepared
         provider methods"""
         if len(event) == 5:  # with special provider
-            kind, date, author, data, provider = event
+            kind, datetime, author, data, provider = event
         else:
-            kind, date, author, data = event
+            kind, datetime, author, data = event
         render = lambda field, context: \
                  provider.render_timeline_event(context, field, event)
-        dateuid = to_utimestamp(date)
-        return {'kind': kind, 'author': author, 'date': date,
-                'dateuid': dateuid, 'render': render, 'event': event,
-                'data': data, 'provider': provider}
-
-    def _provider_failure(self, exc, req, ep, current_filters, all_filters):
-        """Raise a TracError exception explaining the failure of a provider.
-
-        At the same time, the message will contain a link to the timeline
-        without the filters corresponding to the guilty event provider `ep`.
-        """
-        self.log.error("Timeline event provider failed: %s",
-                       exception_to_unicode(exc, traceback=True))
-
-        ep_kinds = dict((f[0], f[1])
-                        for f in ep.get_timeline_filters(req) or [])
-        ep_filters = set(ep_kinds.keys())
-        current_filters = set(current_filters)
-        other_filters = set(current_filters) - ep_filters
-        if not other_filters:
-            other_filters = set(all_filters) - ep_filters
-        args = [(a, req.args.get(a))
-                for a in ('from', 'format', 'max', 'daysback')]
-        href = req.href.timeline(args + [(f, 'on') for f in other_filters])
-        # TRANSLATOR: ...want to see the 'other kinds of events' from... (link)
-        other_events = tag.a(_('other kinds of events'), href=href)
-        raise TracError(tag(
-            tag.p(tag_("Event provider %(name)s failed for filters "
-                       "%(kinds)s: ",
-                       name=tag.code(ep.__class__.__name__),
-                       kinds=', '.join('"%s"' % ep_kinds[f] for f in
-                                       current_filters & ep_filters)),
-                  tag.strong(exception_to_unicode(exc)), class_='message'),
-            tag.p(tag_("You may want to see the %(other_events)s from the "
-                       "Timeline or notify your Trac administrator about the "
-                       "error (detailed information was written to the log).",
-                       other_events=other_events))))
+        localized_datetime = to_datetime(datetime, tzinfo=req.tz)
+        localized_date = truncate_datetime(localized_datetime)
+        datetime_uid = to_utimestamp(localized_datetime)
+        return {'kind': kind, 'author': author, 'date': localized_date,
+                'datetime': localized_datetime, 'datetime_uid': datetime_uid,
+                'render': render,
+                'unread': lastvisit and lastvisit < datetime_uid,
+                'event': event, 'data': data, 'provider': provider}
