@@ -16,11 +16,9 @@
 
 import re
 
-from genshi.builder import Element, Fragment, tag
-
 from trac.config import ConfigSection
 from trac.core import *
-from trac.util.html import find_element
+from trac.util.html import Element, Fragment, find_element, tag
 from trac.util.translation import N_, _, tag_
 from trac.web.api import IRequestHandler
 from trac.wiki.api import IWikiMacroProvider
@@ -35,27 +33,27 @@ class InterTracDispatcher(Component):
     is_valid_default_handler = False
 
     intertrac_section = ConfigSection('intertrac',
-        """This section configures InterTrac prefixes. Options in this section
-        whose name contain a `.` define aspects of the InterTrac prefix
-        corresponding to the option name up to the `.`. Options whose name
-        don't contain a `.` define an alias.
+        """This section configures InterTrac prefixes. Option names in
+        this section that contain a `.` are of the format
+        `<name>.<attribute>`. Option names that don't contain a `.` define
+        an alias.
 
-        The `.url` is mandatory and is used for locating the other Trac.
-        This can be a relative URL in case that Trac environment is located
-        on the same server.
+        The `.url` attribute is mandatory and is used for locating the
+        other Trac. This can be a relative path when the other Trac
+        environment is located on the same server.
 
-        The `.title` information is used for providing a useful tooltip when
-        moving the cursor over an InterTrac link.
+        The `.title` attribute is used for generating a tooltip when the
+        cursor is hovered over an InterTrac link.
 
         Example configuration:
-        {{{
+        {{{#!ini
         [intertrac]
         # -- Example of setting up an alias:
         t = trac
 
         # -- Link to an external Trac:
-        trac.title = Edgewall's Trac for Trac
-        trac.url = https://trac.edgewall.org
+        genshi.title = Edgewall's Trac for Genshi
+        genshi.url = https://genshi.edgewall.org
         }}}
         """)
 
@@ -129,4 +127,4 @@ class InterTracDispatcher(Component):
         return tag.table(class_="wiki intertrac")(
             tag.tr(tag.th(tag.em(_("Prefix"))),
                    tag.th(tag.em(_("Trac Site")))),
-            [generate_prefix(p) for p in sorted(intertracs.keys())])
+            [generate_prefix(p) for p in sorted(intertracs)])
